AUTH_REQUIRED = "Authentication required"
AUTH_EXPIRED = "Your session has expired. Please log in again."
REDIRECT_SEARCH = "/login?next=/search"
FILESHARE_DISABLED_MSG = "File sharing is disabled"
DB_NOT_AVAILABLE_MSG = "Database connection not available"

ACCESS_TOKEN_INVALID_OR_EXPIRED = "Access denied: Invalid or expired access token"
INVALID_SHARE_LINK = (
    "Invalid share link: The requested share does not exist or has been removed"
)
FS_DISABLED_MSG = "File sharing is disabled"
APPLICATION_OCTET_STREAM = "application/octet-stream"


FILES_BASE_URL = "/files/"
LOGIN_HTML = "login.html"
TOO_MANY_LOGIN_ATTEMPTS_MSG = "Too many login attempts. Please try again later."
INVALID_INPUT_LENGTH_MSG = "Invalid input length."
INVALID_USERNAME_OR_PASSWORD_MSG = "Invalid username or password."
ADMIN_URL = "/admin"
ADMIN_LOGIN_TEMPLATE = "admin_login.html"
PROFILE_TEMPLATE = "profile.html"


INVALID_JSON = "Invalid JSON format"
CONNECTION_LIMIT_EXCEEDED = "Connection limit exceeded. Please try again later."
CONNECTION_LIMIT_EXCEEDED_CODE = 1013
CONNECTION_LIMIT_EXCEEDED_REASON = "Connection limit exceeded"
AUTH_FAILED = "Authentication failed. Please try again."
AUTH_FAILED_CODE = 401
AUTH_FAILED_REASON = "Authentication failed"
AUTH_FAILED_MESSAGE = "Authentication failed. Please try again."
AUTH_FAILED_TEMPLATE = "login.html"
AUTH_FAILED_SETTINGS = "settings"
AUTH_FAILED_NEXT_URL = "next_url"
