        body {
            background-color: white;
            color: black;
            font-family: monospace;
            margin: 20px;
        }
        input, select {
            font-family: monospace;
            border: 1px solid black;
            padding: 5px;
            width: 200px;
        }
        .form-group {
            margin: 15px 0;
        }
        .form-group label {
            display: inline-block;
            width: 150px;
            vertical-align: top;
        }
        .section {
            margin: 20px 0;
            padding: 15px;
            border: 1px solid #ccc;
            background-color: #f9f9f9;
        }
        .error {
            color: red;
            margin: 10px 0;
        }
        .nav-links {
            margin-bottom: 20px;
            padding-bottom: 10px;
            border-bottom: 1px solid #ccc;
        }
        .nav-links a {
            margin-right: 15px;
            text-decoration: none;
            color: #007bff;
        }
        .nav-links a:hover {
            text-decoration: underline;
        }
        .btn {
            padding: 8px 16px;
            margin-right: 10px;
            border: 1px solid #ccc;
            background-color: #f8f9fa;
            color: black;
            text-decoration: none;
            display: inline-block;
        }
        .btn:hover {
            background-color: #e9ecef;
        }
        .btn-primary {
            background-color: #007bff;
            color: white;
            border-color: #007bff;
        }
        .btn-primary:hover {
            background-color: #0056b3;
        }
        .help-text {
            font-size: 0.9em;
            color: #666;
            margin-left: 160px;
            margin-top: 5px;
        }
